package org.eclipse.equinox.launcher;

import java.io.IOException;
import java.io.InputStream;
import java.io.FilterInputStream;

public class MultiInputStream extends FilterInputStream
{
    private ByteArrayBuffer buffer;
    
    public MultiInputStream(final InputStream in) {
        super(in);
    }
    
    protected ByteArrayBuffer getBuffer() {
        if (this.buffer == null) {
            this.buffer = new ByteArrayBuffer();
        }
        return this.buffer;
    }
    
    @Override
    public synchronized int read() throws IOException {
        final ByteArrayBuffer buffer = this.getBuffer();
        if (buffer.size() > 0) {
            final int value = buffer.getByteArray()[0];
            buffer.consumed(1);
            return value;
        }
        return super.read();
    }
    
    @Override
    public synchronized int read(final byte[] b, final int off, final int len) throws IOException {
        final ByteArrayBuffer buffer = this.getBuffer();
        if (buffer.size() > 0) {
            final int count = Math.min(buffer.size(), len);
            System.arraycopy(buffer.getByteArray(), 0, b, off, count);
            buffer.consumed(count);
            return count;
        }
        return super.read(b, off, len);
    }
    
    public synchronized void sendMessage(final String message) {
        try {
            System.out.print(message);
            this.getBuffer().write(message.getBytes());
        }
        catch (final IOException ex) {}
    }
    
    @Override
    public int available() throws IOException {
        return super.available() + this.getBuffer().size();
    }
}
